#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <ctype.h>

struct feature_node
{
	int index;
	double value;
};

struct feature_node *x;
int max_nr_attr = 64;

static char *line;
static int max_line_len;

void exit_input_error(int line_num)
{
	fprintf(stderr,"Wrong input format at line %d\n", line_num);
	exit(1);
}
static char* readline(FILE *input)
{
	int len;

	if(fgets(line,max_line_len,input) == NULL)
		return NULL;

	while(strrchr(line,'\n') == NULL)
	{
		max_line_len *= 2;
		line = (char *) realloc(line,(size_t)max_line_len);
		len = (int) strlen(line);
		if(fgets(line+len,max_line_len-len,input) == NULL)
			break;
	}
	return line;
}

void expand_output(int label, struct feature_node *x, int max_idx, FILE *output, double g, double r)
{
	int idxi, idxj;
	int n = max_idx;	
	const struct feature_node *xi=x, *xj;
	double sqrt2, sqrt_2coef0_g, sqrt_2_g;
	sqrt2=sqrt(2);
	sqrt_2coef0_g = sqrt2*sqrt(r*g);
	sqrt_2_g = sqrt2*g;

	fprintf(output, "%g ", (double)label);
	fprintf(output, "%d:%.10lf ", 1, r);
	for(; (idxi=xi->index)!=-1; xi++)
	{
		fprintf(output, "%d:%.10lf ", idxi+1, sqrt_2coef0_g*xi->value);
	}

	xi = x;
	for(; (idxi=xi->index)!=-1; xi++)
	{
		double value = xi->value;
		int w_idx = idxi * (2*n-idxi+1)/2 +1;
		fprintf(output, "%d:%.10lf ", (w_idx + idxi), g*value*value);
		xj = xi + 1;
		for(; (idxj=xj->index)!=-1; xj++)
		{
			fprintf(output, "%d:%.10lf ", (w_idx + idxj), sqrt_2_g*value*xj->value);
		}
	}	
	fprintf(output, "\n");
}

int main(int argc, char **argv){

	FILE *input, *output;
	int total = 0;

	int max_idx = 0;
	int idx_val;
	char *idx;
	if(argc < 4){
		fprintf(stderr,"./expand input_file outpu_file g r");
	}
	input = fopen(argv[1], "r");
	if(input == NULL)
	{
		fprintf(stderr, "cannot open input file %s\n",argv[1]);
		exit(1);
	}
	output = fopen(argv[2], "w");
	if(output == NULL)
	{
		fprintf(stderr, "cannot open output file %s\n",argv[2]);
	}

	max_line_len = 1024;
	line = (char *)malloc(max_line_len*sizeof(char));

	while(readline(input)!=NULL)
	{
		char *p = strtok(line," \t"); // label

		// features
		while(1)
		{
			if(p == NULL || *p == '\n')
				break;			
			idx = strtok(NULL,":");			
			if(idx == NULL || *idx == '\n')
				break;
			idx_val = (int)strtol(idx,NULL,10);
			if(idx_val > max_idx)
				max_idx = idx_val;			
			p = strtok(NULL," \t");
		}
	}
	rewind(input);

	while(readline(input) != NULL)
	{
		int i = 0;
		int target_label;
		char *idx, *val, *label, *endptr;

		x = (struct feature_node *) malloc(max_nr_attr*sizeof(struct feature_node));

		label = strtok(line," \t");
		target_label = (int) strtol(label,&endptr,10);
		if(endptr == label)
			exit_input_error(total+1);

		while(1)
		{
			if(i>=max_nr_attr-2)	// need one more for index = -1
			{
				max_nr_attr *= 2;
				x = (struct feature_node *) realloc(x,max_nr_attr*sizeof(struct feature_node));
			}

			idx = strtok(NULL,":");
			val = strtok(NULL," \t");

			if(val == NULL)
				break;
			errno = 0;
			x[i].index = (int) strtol(idx,&endptr,10);
			if(endptr == idx || errno != 0 || *endptr != '\0' || x[i].index <= 0)
				exit_input_error(total+1);

			errno = 0;
			x[i].value = strtod(val,&endptr);
			if(endptr == val || errno != 0 || (*endptr != '\0' && !isspace(*endptr)))
				exit_input_error(total+1);

			++i;

		}

		x[i].index = -1;

		expand_output(target_label, x, max_idx, output, atof(argv[3]), atof(argv[4]));

	}

	return 0;
}
